'use client'
import { __awaiter } from '../../../../../_virtual/_tslib.js';
import { MFADeviceType } from '@dynamic-labs/sdk-api-core';
import { MfaInvalidOtpError, MfaRateLimitedError } from '@dynamic-labs/utils';
import { sdkApi } from '../api.js';
import { logResponseError } from '../utils.js';

const getUserMfaDevices = (_a) => __awaiter(void 0, [_a], void 0, function* ({ environmentId, }) {
    try {
        const { devices } = yield sdkApi().getUserMfaDevices({
            environmentId,
        });
        return devices;
    }
    catch (e) {
        yield logResponseError(e, 'Error getting mfa devices');
        return [];
    }
});
const addMfaDevice = (_b) => __awaiter(void 0, [_b], void 0, function* ({ environmentId, type, }) {
    try {
        if (type === MFADeviceType.Totp) {
            const response = yield sdkApi().registerTotpMfaDevice({
                environmentId,
            });
            return response;
        }
        throw new Error('Unsupported MFA device type');
    }
    catch (e) {
        const responseError = yield logResponseError(e, 'Error adding mfa device');
        const message = e instanceof Error ? e.message : responseError.error;
        throw new Error(message);
    }
});
const updateUserMfaDevice = (_c) => __awaiter(void 0, [_c], void 0, function* ({ environmentId, mfaDeviceId, }) {
    try {
        yield sdkApi().updateUserMfaDevice({
            environmentId,
            mfaDeviceId,
        });
    }
    catch (e) {
        yield logResponseError(e, 'Error updating mfa device');
    }
});
const deleteMfaDevice = (_d) => __awaiter(void 0, [_d], void 0, function* ({ environmentId, mfaDeviceId, mfaAuthToken, }) {
    try {
        yield sdkApi().deleteMfaDevice({
            environmentId,
            mfaDeviceId,
            xMfaAuthToken: mfaAuthToken,
        });
    }
    catch (e) {
        yield logResponseError(e, 'Error delete mfa device');
    }
});
// used when verifying a device added after the user is already logged in
// no jwt update needed
const verifyMfaDevice = (_e) => __awaiter(void 0, [_e], void 0, function* ({ environmentId, type, code, }) {
    try {
        if (type === MFADeviceType.Totp) {
            const request = {
                code,
                type,
            };
            const device = yield sdkApi().registerTotpMfaDeviceVerify({
                environmentId,
                mFARegisterTotpDevicePostRequest: request,
            });
            return device;
        }
        throw new Error('Unsupported MFA device type');
    }
    catch (e) {
        yield logResponseError(e, 'Error verify mfa device');
        return undefined;
    }
});
// used when verifying a device when user is logging in
// jwt update needed
const authMfaDevice = (_f) => __awaiter(void 0, [_f], void 0, function* ({ deviceId, environmentId, type, code, createMfaToken, }) {
    try {
        if (type === MFADeviceType.Totp) {
            const request = {
                code,
                createMfaToken,
                id: deviceId || undefined,
            };
            const response = yield sdkApi().authMfaTotpDevice({
                environmentId,
                mFAAuthTotpDevicePostRequest: request,
            });
            return response;
        }
        throw new Error('Unsupported MFA device type');
    }
    catch (error) {
        const responseError = yield logResponseError(error, 'Error auth mfa devices');
        if (error instanceof Response) {
            if (responseError.code === 'mfa_invalid_code') {
                throw new MfaInvalidOtpError();
            }
            if (responseError.code === 'mfa_rate_limited') {
                throw new MfaRateLimitedError();
            }
        }
        throw error;
    }
});
const getRecoveryCodes = (_g) => __awaiter(void 0, [_g], void 0, function* ({ environmentId, }) {
    try {
        const { recoveryCodes } = yield sdkApi().getRecoveryCodes({
            environmentId,
        });
        return recoveryCodes;
    }
    catch (e) {
        yield logResponseError(e, 'Error get mfa recovery codes');
        return [];
    }
});
const createNewRecoveryCodes = (_h) => __awaiter(void 0, [_h], void 0, function* ({ environmentId, }) {
    try {
        const { recoveryCodes } = yield sdkApi().createNewRecoveryCodes({
            environmentId,
        });
        return recoveryCodes;
    }
    catch (e) {
        yield logResponseError(e, 'Error create mfa recovery codes');
        return [];
    }
});
// used when verifying a device when user is logging in with recovery code
// jwt update needed
const authMfaRecovery = (_j) => __awaiter(void 0, [_j], void 0, function* ({ environmentId, code, }) {
    try {
        const request = {
            code,
        };
        const response = yield sdkApi().authMfaRecovery({
            environmentId,
            mFAAuthRecoveryDevicePostRequest: request,
        });
        return response;
    }
    catch (error) {
        const responseError = yield logResponseError(error, 'Error mfa recovery');
        if (error instanceof Response) {
            if (responseError.code === 'mfa_invalid_code') {
                throw new MfaInvalidOtpError();
            }
        }
        throw error;
    }
});

export { addMfaDevice, authMfaDevice, authMfaRecovery, createNewRecoveryCodes, deleteMfaDevice, getRecoveryCodes, getUserMfaDevices, updateUserMfaDevice, verifyMfaDevice };
